﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Windows.Media.Animation;
using System.Windows.Threading;
using System.Windows.Controls.Primitives;
using FBNotifier;
using System.Diagnostics;

namespace DesktopAlert
{
    /// <summary>
    /// Follow steps 1a or 1b and then 2 to use this custom control in a XAML file.
    ///
    /// Step 1a) Using this custom control in a XAML file that exists in the current project.
    /// Add this XmlNamespace attribute to the root element of the markup file where it is 
    /// to be used:
    ///
    ///     xmlns:MyNamespace="clr-namespace:DesktopAlert"
    ///
    ///
    /// Step 1b) Using this custom control in a XAML file that exists in a different project.
    /// Add this XmlNamespace attribute to the root element of the markup file where it is 
    /// to be used:
    ///
    ///     xmlns:MyNamespace="clr-namespace:DesktopAlert;assembly=DesktopAlert"
    ///
    /// You will also need to add a project reference from the project where the XAML file lives
    /// to this project and Rebuild to avoid compilation errors:
    ///
    ///     Right click on the target project in the Solution Explorer and
    ///     "Add Reference"->"Projects"->[Browse to and select this project]
    ///
    ///
    /// Step 2)
    /// Go ahead and use your control in the XAML file.
    ///
    ///     <MyNamespace:DesktopAlertBase/>
    ///
    /// </summary>
    public abstract class DesktopAlertBase : Window
    {
        private DoubleAnimation _fadeInAnimation;
        private DoubleAnimation _fadeOutAnimation;
        private DispatcherTimer _activeTimer;
        private DependencyProperty _animationProperty;

        public int TimeOnScreenInSec { get; set; }
        public double FadeTimeInSec { get; set; }
        public string AlertURL { get; set; }

        
        static DesktopAlertBase()
        {
            DefaultStyleKeyProperty.OverrideMetadata(typeof(DesktopAlertBase), new FrameworkPropertyMetadata(typeof(DesktopAlertBase)));
        }

        // We need to add a public constructor so that we can set up our
        // new window properly
        public DesktopAlertBase()
        {
            _animationProperty = (Application.Current as App).AnimationType == App.AnimationTypes.FadingWindow ? DesktopAlertBase.OpacityProperty : DesktopAlertBase.LeftProperty;

            ShowActivated = false;
            Visibility = Visibility.Visible;

            // Let's set up a dummy window size
            Width = 350;
            Height = 75;

            // Set some default properties for the alerts.
            // These can be changed by the derived alerts.
            ShowInTaskbar = false;
            WindowStyle = WindowStyle.None;
            ResizeMode = ResizeMode.NoResize;
            Topmost = true;
            BorderThickness = new Thickness(1);
            BorderBrush = Brushes.Black;
            Background = Brushes.White;
            Opacity = 1.0;

            if (_animationProperty == DesktopAlertBase.OpacityProperty)
            {
                AllowsTransparency = true;
            }
            else
            {
                AllowsTransparency = false;
            }

            Loaded += new RoutedEventHandler(DesktopAlertBase_Loaded);
            MouseEnter += new MouseEventHandler(DesktopAlertBase_MouseEnter);
            MouseLeave += new MouseEventHandler(DesktopAlertBase_MouseLeave);
            MouseDown += new MouseButtonEventHandler(DesktopAlertBase_MouseDown);
        }

        void DesktopAlertBase_MouseDown(object sender, MouseButtonEventArgs e)
        {
            if (AlertURL != null)
            {
                if (AlertURL != "")
                {
                    Process.Start(AlertURL);
                }
            }
        }

        void DesktopAlertBase_MouseLeave(object sender, MouseEventArgs e)
        {
            _activeTimer.Stop();
            _activeTimer.Start();
        }

        void DesktopAlertBase_MouseEnter(object sender, MouseEventArgs e)
        {
            _activeTimer.Stop();
        }

        // When the template is applied to the control, look for a button called "PART_CloseButton".
        // If such a button exists, hook up an event handler so that the alert can be closed when
        // the user clicks the button.
        public override void OnApplyTemplate()
        {
            ButtonBase closeButton = Template.FindName("PART_CloseButton", this) as ButtonBase;
            if (closeButton != null)
                closeButton.Click += new RoutedEventHandler(closeButton_Click);
        }

        // In the Loaded-event handler we need to figure out where to place the alert window.
        // Currently they are all placed in the bottom right corner, and can not take into
        // account other currently open alert windows.
        virtual protected void DesktopAlertBase_Loaded(object sender, RoutedEventArgs e)
        {
            // Set up the fade in and fade out animations
            _fadeInAnimation = new DoubleAnimation();
            if (_animationProperty == DesktopAlertBase.LeftProperty)
            {
                _fadeInAnimation.From = SystemParameters.WorkArea.Width;
                _fadeInAnimation.To = SystemParameters.WorkArea.Width - Width;
            }
            else if (_animationProperty == DesktopAlertBase.OpacityProperty)
            {
                _fadeInAnimation.From = 0;
                _fadeInAnimation.To = 1;
            }
            _fadeInAnimation.Duration = new Duration(TimeSpan.FromSeconds(FadeTimeInSec));

            // For the fade out we omit the from, so that it can be smoothly initiated
            // from a fade in that gets interrupted when the user wants to close the window
            _fadeOutAnimation = new DoubleAnimation();
            if (_animationProperty == DesktopAlertBase.LeftProperty)
            {
                _fadeOutAnimation.From = SystemParameters.WorkArea.Width - Width;
                _fadeOutAnimation.To = SystemParameters.WorkArea.Width;

                // Figure out where to place the window based on the current screen resolution
                Left = System.Windows.SystemParameters.WorkArea.Right;
                Top = System.Windows.SystemParameters.WorkArea.Bottom - Height;
            }
            else if (_animationProperty == DesktopAlertBase.OpacityProperty)
            {
                _fadeOutAnimation.From = 1;
                _fadeOutAnimation.To = 0;

                // Figure out where to place the window based on the current screen resolution
                Left = System.Windows.SystemParameters.WorkArea.Right - Width;
                Top = System.Windows.SystemParameters.WorkArea.Bottom - Height;
            }
            _fadeOutAnimation.Duration = new Duration(TimeSpan.FromSeconds(FadeTimeInSec));

            _activeTimer = new DispatcherTimer();
            _activeTimer.Interval = new TimeSpan(0, 0, TimeOnScreenInSec);

            // Attach an anonymous method to the timer so that we can start fading out the alert
            // when the timer is done.
            _activeTimer.Tick += ActiveTimer_Completed;

            _fadeInAnimation.Completed += FadeInAnimation_Completed;

            // Start the fade in animation
            BeginAnimation(_animationProperty, _fadeInAnimation);
        }

        // When the fade in animation is completed, start another timer that fires an event when the
        // window has been visible for 10 seconds
        protected virtual void FadeInAnimation_Completed(object sender, EventArgs e)
        {
            _activeTimer.Start();
        }

        protected virtual void FadeOutAnimation_Completed(object sender, EventArgs e)
        {
            Close();
        }

        protected virtual void ActiveTimer_Completed(object sender, EventArgs e)
        {
            _activeTimer.Stop(); 
            FadeOut();
        }

        // Set up the fade out animation, and hook up an event handler to fire when it is completed.
        private void FadeOut()
        {
            // Attach an anonymous method to the Completed-event of the fade out animation
            // so that we can close the alert window when the animation is done.
            _fadeOutAnimation.Completed += FadeOutAnimation_Completed;

            BeginAnimation(_animationProperty, _fadeOutAnimation, HandoffBehavior.SnapshotAndReplace);
        }

        // If the user clicks the close button, stop the timer that counts how long the alert
        // window has been open, and start the fading out of the window.
        void closeButton_Click(object sender, RoutedEventArgs e)
        {
            if (_activeTimer != null)
            {
                _activeTimer.Stop();
            }
            FadeOut();
        }
    }
}
